@@ -351,8 +351,7 @@ class Shade(_Base):
         Returns:
             True if geometrically equivalent. False if not geometrically equivalent.
         """
-        tol_val = tolerance * self.area
-        if not shade.area - tol_val <= self.area <= shade.area + tol_val:
+        if abs(self.area - shade.area) > tolerance * self.area:
             return False
         return self.geometry.is_centered_adjacent(shade.geometry, tolerance)
 
